@@ -1,6 +1,7 @@
 #pragma once
 
 #include "modules/meta/event_module.hpp"
+#include "modules/meta/types.hpp"
 #include "settings.hpp"
 
 POLYBAR_NS
@@ -13,7 +14,7 @@ namespace modules {
 
   class pulseaudio_module : public event_module<pulseaudio_module> {
    public:
-    explicit pulseaudio_module(const bar_settings&, string);
+    explicit pulseaudio_module(const bar_settings&, string, const config&);
 
     void teardown();
     bool has_event();
@@ -22,7 +23,7 @@ namespace modules {
     string get_output();
     bool build(builder* builder, const string& tag) const;
 
-    static constexpr auto TYPE = "internal/pulseaudio";
+    static constexpr auto TYPE = PULSEAUDIO_TYPE;
 
     static constexpr auto EVENT_INC = "inc";
     static constexpr auto EVENT_DEC = "dec";
@@ -53,6 +54,7 @@ namespace modules {
     atomic<bool> m_muted{false};
     atomic<int> m_volume{0};
     atomic<double> m_decibels{0};
+    atomic<bool> m_reverse_scroll{false};
   };
 }  // namespace modules
 
