@@ -13,6 +13,7 @@
 #include "utils/bspwm.hpp"
 #include "utils/color.hpp"
 #include "utils/math.hpp"
+#include "utils/restack.hpp"
 #include "utils/string.hpp"
 #include "utils/units.hpp"
 #include "x11/atoms.hpp"
@@ -20,7 +21,7 @@
 #include "x11/ewmh.hpp"
 #include "x11/extensions/all.hpp"
 #include "x11/icccm.hpp"
-#include "x11/tray_manager.hpp"
+#include "x11/legacy_tray_manager.hpp"
 
 #if WITH_XCURSOR
 #include "x11/cursor.hpp"
@@ -38,18 +39,17 @@ using namespace eventloop;
 /**
  * Create instance
  */
-bar::make_type bar::make(loop& loop, bool only_initialize_values) {
+bar::make_type bar::make(loop& loop, const config& config, bool only_initialize_values) {
   auto action_ctxt = make_unique<tags::action_context>();
 
   // clang-format off
   return std::make_unique<bar>(
       connection::make(),
       signal_emitter::make(),
-      config::make(),
+      config,
       logger::make(),
       loop,
-      screen::make(),
-      tray_manager::make(),
+      screen::make(config),
       tags::dispatch::make(*action_ctxt),
       std::move(action_ctxt),
       only_initialize_values);
@@ -58,29 +58,28 @@ bar::make_type bar::make(loop& loop, bool only_initialize_values) {
 
 /**
  * Construct bar instance
- *
- * TODO: Break out all tray handling
  */
 bar::bar(connection& conn, signal_emitter& emitter, const config& config, const logger& logger, loop& loop,
-    unique_ptr<screen>&& screen, unique_ptr<tray_manager>&& tray_manager, unique_ptr<tags::dispatch>&& dispatch,
-    unique_ptr<tags::action_context>&& action_ctxt, bool only_initialize_values)
+    unique_ptr<screen>&& screen, unique_ptr<tags::dispatch>&& dispatch, unique_ptr<tags::action_context>&& action_ctxt,
+    bool only_initialize_values)
     : m_connection(conn)
     , m_sig(emitter)
     , m_conf(config)
     , m_log(logger)
     , m_loop(loop)
     , m_screen(forward<decltype(screen)>(screen))
-    , m_tray(forward<decltype(tray_manager)>(tray_manager))
     , m_dispatch(forward<decltype(dispatch)>(dispatch))
     , m_action_ctxt(forward<decltype(action_ctxt)>(action_ctxt)) {
   string bs{m_conf.section()};
 
+  m_tray = legacy_tray::tray_manager::make(m_opts);
+
   // Get available RandR outputs
   auto monitor_name = m_conf.get(bs, "monitor", ""s);
   auto monitor_name_fallback = m_conf.get(bs, "monitor-fallback", ""s);
   m_opts.monitor_strict = m_conf.get(bs, "monitor-strict", m_opts.monitor_strict);
   m_opts.monitor_exact = m_conf.get(bs, "monitor-exact", m_opts.monitor_exact);
-  auto monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, m_opts.monitor_strict, false);
+  auto monitors = randr_util::get_monitors(m_connection, m_opts.monitor_strict, false);
 
   if (monitors.empty()) {
     throw application_error("No monitors found");
@@ -98,7 +97,7 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
 
   // if still not found (and not strict matching), get first connected monitor
   if (monitor_name.empty() && !m_opts.monitor_strict) {
-    auto connected_monitors = randr_util::get_monitors(m_connection, m_connection.screen()->root, true, false);
+    auto connected_monitors = randr_util::get_monitors(m_connection, true, false);
     if (!connected_monitors.empty()) {
       monitor_name = connected_monitors[0]->name;
       m_log.warn("No monitor specified, using \"%s\"", monitor_name);
@@ -131,12 +130,7 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
   m_log.info("Loaded monitor %s (%ix%i+%i+%i)", m_opts.monitor->name, m_opts.monitor->w, m_opts.monitor->h,
       m_opts.monitor->x, m_opts.monitor->y);
 
-  try {
-    m_opts.override_redirect = m_conf.get<bool>(bs, "dock");
-    m_conf.warn_deprecated(bs, "dock", "override-redirect");
-  } catch (const key_error& err) {
-    m_opts.override_redirect = m_conf.get(bs, "override-redirect", m_opts.override_redirect);
-  }
+  m_opts.override_redirect = m_conf.deprecated(bs, "dock", "override-redirect", m_opts.override_redirect);
 
   m_opts.dimvalue = m_conf.get(bs, "dim-value", 1.0);
   m_opts.dimvalue = math_util::cap(m_opts.dimvalue, 0.0, 1.0);
@@ -223,11 +217,22 @@ bar::bar(connection& conn, signal_emitter& emitter, const config& config, const
 
   m_opts.double_click_interval = m_conf.get(bs, "double-click-interval", m_opts.double_click_interval);
 
+  m_opts.struts = m_conf.get(bs, "enable-struts", m_opts.struts);
+
   if (only_initialize_values) {
     return;
   }
 
   // Load values used to adjust the struts atom
+
+  if (!m_opts.struts) {
+    if (m_conf.has("global/wm", "margin-bottom")) {
+      m_log.warn("Struts are disabled, ignoring margin-bottom");
+    }
+    if (m_conf.has("global/wm", "margin-top")) {
+      m_log.warn("Struts are disabled, ignoring margin-top");
+    }
+  }
   auto margin_top = m_conf.get("global/wm", "margin-top", percentage_with_offset{});
   auto margin_bottom = m_conf.get("global/wm", "margin-bottom", percentage_with_offset{});
   m_opts.strut.top = units_utils::percentage_with_offset_to_pixel(margin_top, m_opts.monitor->h, m_opts.dpi_y);
@@ -394,12 +399,12 @@ void bar::parse(string&& data, bool force) {
   auto rect = m_opts.inner_area();
 
   if (m_tray && !m_tray->settings().detached && m_tray->settings().configured_slots) {
-    auto trayalign = m_tray->settings().align;
+    auto tray_pos = m_tray->settings().tray_position;
     auto traywidth = m_tray->settings().configured_w;
-    if (trayalign == alignment::LEFT) {
+    if (tray_pos == legacy_tray::tray_postition::LEFT) {
       rect.x += traywidth;
       rect.width -= traywidth;
-    } else if (trayalign == alignment::RIGHT) {
+    } else if (tray_pos == legacy_tray::tray_postition::RIGHT) {
       rect.width -= traywidth;
     }
   }
@@ -415,19 +420,12 @@ void bar::parse(string&& data, bool force) {
 
   m_renderer->end();
 
-  const auto check_dblclicks = [&]() -> bool {
-    if (m_action_ctxt->has_double_click()) {
-      return true;
+  m_dblclicks.clear();
+  for (auto&& action : m_opts.actions) {
+    if (static_cast<int>(action.button) >= static_cast<int>(mousebtn::DOUBLE_LEFT)) {
+      m_dblclicks.insert(action.button);
     }
-
-    for (auto&& action : m_opts.actions) {
-      if (static_cast<int>(action.button) >= static_cast<int>(mousebtn::DOUBLE_LEFT)) {
-        return true;
-      }
-    }
-    return false;
-  };
-  m_dblclicks = check_dblclicks();
+  }
 }
 
 /**
@@ -443,7 +441,7 @@ void bar::hide() {
     m_visible = false;
     reconfigure_struts();
     m_sig.emit(visibility_change{false});
-    m_connection.unmap_window_checked(m_opts.window);
+    m_connection.unmap_window_checked(m_opts.x_data.window);
     m_connection.flush();
   } catch (const exception& err) {
     m_log.err("Failed to unmap bar window (err=%s", err.what());
@@ -482,8 +480,11 @@ void bar::toggle() {
 }
 
 /**
- * Move the bar window above defined sibling
- * in the X window stack
+ * Move the bar window above/below defined sibling in the X window stack.
+ *
+ * How the sibling is determined depends on the wm-restack setting.
+ *
+ * This is meant to resolve polybar appearing above other windows (especially in fullscreen).
  */
 void bar::restack_window() {
   string wm_restack;
@@ -494,25 +495,21 @@ void bar::restack_window() {
     return;
   }
 
-  auto restacked = false;
+  xcb_window_t bar_window = m_opts.x_data.window;
+
+  restack_util::params restack_params;
 
   if (wm_restack == "generic") {
-    try {
-      auto children = m_connection.query_tree(m_connection.screen()->root).children();
-      if (children.begin() != children.end() && *children.begin() != m_opts.window) {
-        const unsigned int value_mask = XCB_CONFIG_WINDOW_SIBLING | XCB_CONFIG_WINDOW_STACK_MODE;
-        const unsigned int value_list[2]{*children.begin(), XCB_STACK_MODE_BELOW};
-        m_connection.configure_window_checked(m_opts.window, value_mask, value_list);
-      }
-      restacked = true;
-    } catch (const exception& err) {
-      m_log.err("Failed to restack bar window (err=%s)", err.what());
-    }
+    restack_params = restack_util::get_generic_params(m_connection, bar_window);
+  } else if (wm_restack == "ewmh") {
+    restack_params = restack_util::get_ewmh_params(m_connection);
+  } else if (wm_restack == "bottom") {
+    restack_params = restack_util::get_bottom_params(m_connection, bar_window);
   } else if (wm_restack == "bspwm") {
-    restacked = bspwm_util::restack_to_root(m_connection, m_opts.monitor, m_opts.window);
+    restack_params = bspwm_util::get_restack_params(m_connection);
 #if ENABLE_I3
   } else if (wm_restack == "i3" && m_opts.override_redirect) {
-    restacked = i3_util::restack_to_root(m_connection, m_opts.window);
+    restack_params = i3_util::get_restack_params(m_connection);
   } else if (wm_restack == "i3" && !m_opts.override_redirect) {
     m_log.warn("Ignoring restack of i3 window (not needed when `override-redirect = false`)");
     wm_restack.clear();
@@ -522,10 +519,19 @@ void bar::restack_window() {
     wm_restack.clear();
   }
 
-  if (restacked) {
-    m_log.info("Successfully restacked bar window");
+  auto [restack_sibling, stack_mode] = restack_params;
+
+  if (restack_sibling != XCB_NONE) {
+    try {
+      m_log.info("bar: Restacking bar window relative to %s with stacking mode %s", m_connection.id(restack_sibling),
+          restack_util::stack_mode_to_string(stack_mode));
+      restack_util::restack_relative(m_connection, bar_window, restack_sibling, stack_mode);
+      m_log.info("Successfully restacked bar window");
+    } catch (const exception& err) {
+      m_log.err("Failed to restack bar window (err=%s)", err.what());
+    }
   } else if (!wm_restack.empty()) {
-    m_log.err("Failed to restack bar window");
+    m_log.err("Failed to restack bar window: No suitable sibling window for restacking was found");
   }
 }
 
@@ -541,7 +547,7 @@ void bar::reconfigure_window() {
  * Reconfigure window geometry
  */
 void bar::reconfigure_geom() {
-  window win{m_connection, m_opts.window};
+  window win{m_connection, m_opts.x_data.window};
   win.reconfigure_geom(m_opts.size.w, m_opts.size.h, m_opts.pos.x, m_opts.pos.y);
 }
 
@@ -549,7 +555,7 @@ void bar::reconfigure_geom() {
  * Reconfigure window position
  */
 void bar::reconfigure_pos() {
-  window win{m_connection, m_opts.window};
+  window win{m_connection, m_opts.x_data.window};
   win.reconfigure_pos(m_opts.pos.x, m_opts.pos.y);
 }
 
@@ -557,9 +563,12 @@ void bar::reconfigure_pos() {
  * Reconfigure window strut values
  */
 void bar::reconfigure_struts() {
-  window win{m_connection, m_opts.window};
+  if (!m_opts.struts) {
+    return;
+  }
+  window win{m_connection, m_opts.x_data.window};
   if (m_visible) {
-    auto geom = m_connection.get_geometry(m_screen->root());
+    auto geom = m_connection.get_geometry(m_connection.root());
     int h = m_opts.size.h + m_opts.offset.y;
 
     // Apply user-defined margins
@@ -603,7 +612,7 @@ void bar::reconfigure_struts() {
  * Reconfigure window wm hint values
  */
 void bar::reconfigure_wm_hints() {
-  const auto& win = m_opts.window;
+  const auto& win = m_opts.x_data.window;
 
   m_log.trace("bar: Set window WM_NAME");
   icccm_util::set_wm_name(m_connection, win, m_opts.wmname.c_str(), m_opts.wmname.size(), "polybar\0Polybar", 15_z);
@@ -627,7 +636,7 @@ void bar::reconfigure_wm_hints() {
  * Broadcast current map state
  */
 void bar::broadcast_visibility() {
-  auto attr = m_connection.get_window_attributes(m_opts.window);
+  auto attr = m_connection.get_window_attributes(m_opts.x_data.window);
 
   if (attr->map_state == XCB_MAP_STATE_UNVIEWABLE) {
     m_sig.emit(visibility_change{false});
@@ -648,7 +657,7 @@ void bar::map_window() {
   reconfigure_window();
 
   m_log.trace("bar: Map window");
-  m_connection.map_window_checked(m_opts.window);
+  m_connection.map_window_checked(m_opts.x_data.window);
 
   /*
    * Required by AwesomeWM. AwesomeWM does not seem to respect polybar's position if WM_NORMAL_HINTS are set before
@@ -680,7 +689,7 @@ void bar::trigger_click(mousebtn btn, int pos) {
  * Event handler for XCB_DESTROY_NOTIFY events
  */
 void bar::handle(const evt::client_message& evt) {
-  if (evt->type == WM_PROTOCOLS && evt->data.data32[0] == WM_DELETE_WINDOW && evt->window == m_opts.window) {
+  if (evt->type == WM_PROTOCOLS && evt->data.data32[0] == WM_DELETE_WINDOW && evt->window == m_opts.x_data.window) {
     m_log.err("Bar window has been destroyed, shutting down...");
     m_connection.disconnect();
   }
@@ -690,7 +699,7 @@ void bar::handle(const evt::client_message& evt) {
  * Event handler for XCB_DESTROY_NOTIFY events
  */
 void bar::handle(const evt::destroy_notify& evt) {
-  if (evt->window == m_opts.window) {
+  if (evt->window == m_opts.x_data.window) {
     m_connection.disconnect();
   }
 }
@@ -703,12 +712,12 @@ void bar::handle(const evt::destroy_notify& evt) {
  */
 void bar::handle(const evt::enter_notify&) {
   if (m_opts.dimmed) {
-    m_dim_timer.start(25, 0, [this]() {
+    m_dim_timer->start(25, 0, [this]() {
       m_opts.dimmed = false;
       m_sig.emit(dim_window{1.0});
     });
-  } else if (m_dim_timer.is_active()) {
-    m_dim_timer.stop();
+  } else if (m_dim_timer->is_active()) {
+    m_dim_timer->stop();
   }
 }
 
@@ -722,7 +731,7 @@ void bar::handle(const evt::leave_notify&) {
   // Only trigger dimming, if the dim-value is not fully opaque.
   if (m_opts.dimvalue < 1.0) {
     if (!m_opts.dimmed) {
-      m_dim_timer.start(3000, 0, [this]() {
+      m_dim_timer->start(3000, 0, [this]() {
         m_opts.dimmed = true;
         m_sig.emit(dim_window{double(m_opts.dimvalue)});
       });
@@ -823,16 +832,19 @@ void bar::handle(const evt::button_press& evt) {
     }
   };
 
+  mousebtn double_btn = mousebtn_get_double(btn);
+  bool has_dblclick = m_dblclicks.count(double_btn) || m_action_ctxt->has_action(double_btn, pos) != tags::NO_ACTION;
+
   // If there are no double click handlers defined we can
   // just by-pass the click timer handling
-  if (!m_dblclicks) {
+  if (!has_dblclick) {
     trigger_click(btn, pos);
   } else if (btn == mousebtn::LEFT) {
-    check_double(m_leftclick_timer, btn, pos);
+    check_double(*m_leftclick_timer, btn, pos);
   } else if (btn == mousebtn::MIDDLE) {
-    check_double(m_middleclick_timer, btn, pos);
+    check_double(*m_middleclick_timer, btn, pos);
   } else if (btn == mousebtn::RIGHT) {
-    check_double(m_rightclick_timer, btn, pos);
+    check_double(*m_rightclick_timer, btn, pos);
   } else {
     trigger_click(btn, pos);
   }
@@ -844,8 +856,8 @@ void bar::handle(const evt::button_press& evt) {
  * Used to redraw the bar
  */
 void bar::handle(const evt::expose& evt) {
-  if (evt->window == m_opts.window && evt->count == 0) {
-    if (m_tray->settings().running) {
+  if (evt->window == m_opts.x_data.window && evt->count == 0) {
+    if (m_tray && m_tray->settings().running) {
       broadcast_visibility();
     }
 
@@ -869,34 +881,40 @@ void bar::handle(const evt::property_notify& evt) {
   m_log.trace_x("bar: property_notify(%s)", atom_name);
 #endif
 
-  if (evt->window == m_opts.window && evt->atom == WM_STATE) {
+  if (evt->window == m_opts.x_data.window && evt->atom == WM_STATE) {
     broadcast_visibility();
   }
 }
 
-void bar::handle(const evt::configure_notify&) {
+void bar::handle(const evt::configure_notify& evt) {
+  if (evt->window != m_opts.x_data.window) {
+    return;
+  }
   // The absolute position of the window in the root may be different after configuration is done
   // (for example, because the parent is not positioned at 0/0 in the root window).
   // Notify components that the geometry may have changed (used by the background manager for example).
   m_sig.emit(signals::ui::update_geometry{});
 }
 
-void bar::start() {
+void bar::start(const string& tray_module_name) {
   m_log.trace("bar: Create renderer");
-  m_renderer = renderer::make(m_opts, *m_action_ctxt);
-  m_opts.window = m_renderer->window();
+  m_renderer = renderer::make(m_opts, *m_action_ctxt, m_conf);
+
+  m_opts.x_data.window = m_renderer->window();
+  m_opts.x_data.visual = m_renderer->visual();
+  m_opts.x_data.depth = m_renderer->depth();
 
   // Subscribe to window enter and leave events
   // if we should dim the window
   if (m_opts.dimvalue != 1.0) {
-    m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_ENTER_WINDOW | XCB_EVENT_MASK_LEAVE_WINDOW);
+    m_connection.ensure_event_mask(m_opts.x_data.window, XCB_EVENT_MASK_ENTER_WINDOW | XCB_EVENT_MASK_LEAVE_WINDOW);
   }
   if (!m_opts.cursor_click.empty() || !m_opts.cursor_scroll.empty()) {
-    m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_POINTER_MOTION);
+    m_connection.ensure_event_mask(m_opts.x_data.window, XCB_EVENT_MASK_POINTER_MOTION);
   }
-  m_connection.ensure_event_mask(m_opts.window, XCB_EVENT_MASK_STRUCTURE_NOTIFY);
+  m_connection.ensure_event_mask(m_opts.x_data.window, XCB_EVENT_MASK_STRUCTURE_NOTIFY);
 
-  m_log.info("Bar window: %s", m_connection.id(m_opts.window));
+  m_log.info("Bar window: %s", m_connection.id(m_opts.x_data.window));
 
   map_window();
 
@@ -909,14 +927,19 @@ void bar::start() {
   m_renderer->end();
 
   m_log.trace("bar: Setup tray manager");
-  m_tray->setup(static_cast<const bar_settings&>(m_opts));
+  m_tray->setup(m_conf, tray_module_name);
+
+  if (m_tray->settings().tray_position == legacy_tray::tray_postition::MODULE ||
+      m_tray->settings().tray_position == legacy_tray::tray_postition::NONE) {
+    m_tray.reset();
+  }
 
   broadcast_visibility();
 }
 
 bool bar::on(const signals::ui::dim_window& sig) {
   m_opts.dimmed = sig.cast() != 1.0;
-  ewmh_util::set_wm_window_opacity(m_opts.window, sig.cast() * 0xFFFFFFFF);
+  ewmh_util::set_wm_window_opacity(m_opts.x_data.window, sig.cast() * 0xFFFFFFFF);
   return false;
 }
 
@@ -928,7 +951,7 @@ void bar::change_cursor(const string& name) {
   }
   m_cursor = name;
 
-  if (!cursor_util::set_cursor(m_connection, m_connection.screen(), m_opts.window, name)) {
+  if (!cursor_util::set_cursor(m_connection, m_connection.screen(), m_opts.x_data.window, name)) {
     m_log.warn("Failed to create cursor context for cursor name '%s'", name);
   }
   m_connection.flush();
