@@ -26,8 +26,13 @@ namespace modules {
    * Bootstrap the module by reading config values and
    * setting up required components
    */
-  fs_module::fs_module(const bar_settings& bar, string name_) : timer_module<fs_module>(bar, move(name_)) {
-    m_mountpoints = m_conf.get_list(name(), "mount");
+  fs_module::fs_module(const bar_settings& bar, string name_, const config& config)
+      : timer_module<fs_module>(bar, move(name_), config) {
+    m_mountpoints = m_conf.get_list(name(), "mount", {});
+    if (m_mountpoints.empty()) {
+      m_log.info("%s: No mountpoints specified, using fallback \"/\"", name());
+      m_mountpoints.emplace_back("/");
+    }
     m_remove_unmounted = m_conf.get(name(), "remove-unmounted", m_remove_unmounted);
     m_perc_used_warn = m_conf.get(name(), "warn-percentage", 90);
     m_fixed = m_conf.get(name(), "fixed-values", m_fixed);
