@@ -11,7 +11,8 @@ POLYBAR_NS
 namespace modules {
   template class module<menu_module>;
 
-  menu_module::menu_module(const bar_settings& bar, string name_) : static_module<menu_module>(bar, move(name_)) {
+  menu_module::menu_module(const bar_settings& bar, string name_, const config& config)
+      : static_module<menu_module>(bar, move(name_), config) {
     m_expand_right = m_conf.get(name(), "expand-right", m_expand_right);
 
     m_router->register_action_with_data(EVENT_OPEN, [this](const std::string& data) { action_open(data); });
