@@ -15,17 +15,17 @@ namespace modules {
       builder->flush();
       return "";
     }
-    if (offset != 0) {
+    if (offset) {
       builder->offset(offset);
     }
-    if (margin > 0) {
-      builder->space(margin);
+    if (margin) {
+      builder->spacing(margin);
     }
     if (bg.has_color()) {
       builder->background(bg);
     }
     if (fg.has_color()) {
-      builder->color(fg);
+      builder->foreground(fg);
     }
     if (ul.has_color()) {
       builder->underline(ul);
@@ -36,8 +36,8 @@ namespace modules {
     if (font > 0) {
       builder->font(font);
     }
-    if (padding > 0) {
-      builder->space(padding);
+    if (padding) {
+      builder->spacing(padding);
     }
 
     builder->node(prefix);
@@ -46,7 +46,7 @@ namespace modules {
       builder->background(bg);
     }
     if (fg.has_color()) {
-      builder->color(fg);
+      builder->foreground(fg);
     }
     if (ul.has_color()) {
       builder->underline(ul);
@@ -55,11 +55,11 @@ namespace modules {
       builder->overline(ol);
     }
 
-    builder->append(move(output));
+    builder->node(output);
     builder->node(suffix);
 
-    if (padding > 0) {
-      builder->space(padding);
+    if (padding) {
+      builder->spacing(padding);
     }
     if (font > 0) {
       builder->font_close();
@@ -71,13 +71,13 @@ namespace modules {
       builder->underline_close();
     }
     if (fg.has_color()) {
-      builder->color_close();
+      builder->foreground_close();
     }
     if (bg.has_color()) {
       builder->background_close();
     }
-    if (margin > 0) {
-      builder->space(margin);
+    if (margin) {
+      builder->spacing(margin);
     }
 
     return builder->flush();
@@ -86,13 +86,13 @@ namespace modules {
   // }}}
   // module_formatter {{{
 
-  void module_formatter::add(string name, string fallback, vector<string>&& tags, vector<string>&& whitelist) {
+  void module_formatter::add_value(string&& name, string&& value, vector<string>&& tags, vector<string>&& whitelist) {
     const auto formatdef = [&](const string& param, const auto& fallback) {
       return m_conf.get("settings", "format-" + param, fallback);
     };
 
     auto format = make_unique<module_format>();
-    format->value = m_conf.get(m_modname, name, move(fallback));
+    format->value = std::move(value);
     format->fg = m_conf.get(m_modname, name + "-foreground", formatdef("foreground", format->fg));
     format->bg = m_conf.get(m_modname, name + "-background", formatdef("background", format->bg));
     format->ul = m_conf.get(m_modname, name + "-underline", formatdef("underline", format->ul));
@@ -104,7 +104,6 @@ namespace modules {
     format->margin = m_conf.get(m_modname, name + "-margin", formatdef("margin", format->margin));
     format->offset = m_conf.get(m_modname, name + "-offset", formatdef("offset", format->offset));
     format->font = m_conf.get(m_modname, name + "-font", formatdef("font", format->font));
-    format->tags.swap(tags);
 
     try {
       format->prefix = load_label(m_conf, m_modname, name + "-prefix");
@@ -119,32 +118,39 @@ namespace modules {
     }
 
     vector<string> tag_collection;
-    tag_collection.reserve(format->tags.size() + whitelist.size());
-    tag_collection.insert(tag_collection.end(), format->tags.begin(), format->tags.end());
+    tag_collection.reserve(tags.size() + whitelist.size());
+    tag_collection.insert(tag_collection.end(), tags.begin(), tags.end());
     tag_collection.insert(tag_collection.end(), whitelist.begin(), whitelist.end());
 
-    size_t start, end;
-    string value{format->value};
-    while ((start = value.find('<')) != string::npos && (end = value.find('>', start)) != string::npos) {
-      if (start > 0) {
-        value.erase(0, start);
-        end -= start;
-        start = 0;
-      }
-      string tag{value.substr(start, end + 1)};
+    size_t start = 0;
+    size_t end = 0;
+    while ((start = format->value.find('<', start)) != string::npos && (end = format->value.find('>', start)) != string::npos) {
+      string tag{format->value.substr(start, end - start + 1)};
       if (find(tag_collection.begin(), tag_collection.end(), tag) == tag_collection.end()) {
         throw undefined_format_tag(tag + " is not a valid format tag for \"" + name + "\"");
       }
-      value.erase(0, tag.size());
+      start = end + 1;
     }
 
     m_formats.insert(make_pair(move(name), move(format)));
   }
 
+  void module_formatter::add(string name, string fallback, vector<string>&& tags, vector<string>&& whitelist) {
+    string value = m_conf.get(m_modname, name, move(fallback));
+    add_value(move(name), move(value), forward<vector<string>>(tags), forward<vector<string>>(whitelist));
+  }
+
+  void module_formatter::add_optional(string name, vector<string>&& tags, vector<string>&& whitelist) {
+    if (m_conf.has(m_modname, name)) {
+      string value = m_conf.get(m_modname, name);
+      add_value(move(name), move(value), move(tags), move(whitelist));
+    }
+  }
+
   bool module_formatter::has(const string& tag, const string& format_name) {
     auto format = m_formats.find(format_name);
     if (format == m_formats.end()) {
-      throw undefined_format(format_name);
+      return false;
     }
     return format->second->value.find(tag) != string::npos;
   }
@@ -158,6 +164,10 @@ namespace modules {
     return false;
   }
 
+  bool module_formatter::has_format(const string& format_name) {
+    return m_formats.find(format_name) != m_formats.end();
+  }
+
   shared_ptr<module_format> module_formatter::get(const string& format_name) {
     auto format = m_formats.find(format_name);
     if (format == m_formats.end()) {
@@ -167,6 +177,6 @@ namespace modules {
   }
 
   // }}}
-}  // namespace modules
+} // namespace modules
 
 POLYBAR_NS_END
