@@ -24,6 +24,21 @@ inotify_watch::~inotify_watch() {
   }
 }
 
+inotify_watch::inotify_watch(inotify_watch&& other) noexcept {
+  std::swap(m_path, other.m_path);
+  std::swap(m_wd, other.m_wd);
+  std::swap(m_fd, other.m_fd);
+  std::swap(m_mask, other.m_mask);
+}
+
+inotify_watch& inotify_watch::operator=(inotify_watch&& other) noexcept {
+  std::swap(m_path, other.m_path);
+  std::swap(m_wd, other.m_wd);
+  std::swap(m_fd, other.m_fd);
+  std::swap(m_mask, other.m_mask);
+  return *this;
+}
+
 /**
  * Attach inotify watch
  */
@@ -70,13 +85,15 @@ bool inotify_watch::poll(int wait_ms) const {
 /**
  * Get the latest inotify event
  */
-unique_ptr<inotify_event> inotify_watch::get_event() const {
-  auto event = std::make_unique<inotify_event>();
+inotify_event inotify_watch::get_event() const {
+  inotify_event event;
 
   if (m_fd == -1 || m_wd == -1) {
     return event;
   }
 
+  event.is_valid = true;
+
   char buffer[1024];
   auto bytes = read(m_fd, buffer, 1024);
   auto len = 0;
@@ -84,11 +101,11 @@ unique_ptr<inotify_event> inotify_watch::get_event() const {
   while (len < bytes) {
     auto* e = reinterpret_cast<::inotify_event*>(&buffer[len]);
 
-    event->filename = e->len ? e->name : m_path;
-    event->wd = e->wd;
-    event->cookie = e->cookie;
-    event->is_dir = e->mask & IN_ISDIR;
-    event->mask |= e->mask;
+    event.filename = e->len ? e->name : m_path;
+    event.wd = e->wd;
+    event.cookie = e->cookie;
+    event.is_dir = e->mask & IN_ISDIR;
+    event.mask |= e->mask;
 
     len += sizeof(*e) + e->len;
   }
@@ -96,18 +113,10 @@ unique_ptr<inotify_event> inotify_watch::get_event() const {
   return event;
 }
 
-/**
- * Wait for matching event
- */
-unique_ptr<inotify_event> inotify_watch::await_match() const {
-  auto event = get_event();
-  return event->mask & m_mask ? std::move(event) : nullptr;
-}
-
 /**
  * Get watch file path
  */
-const string inotify_watch::path() const {
+string inotify_watch::path() const {
   return m_path;
 }
 
